<?php
/**
 * @file
 * Contains \Drupal\droogle\DroogleController.
 */
 
namespace Drupal\droogle\Controller;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\Query\QueryFactory;
use Drupal\Core\Extension\ThemeHandlerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Menu\MenuLinkTreeInterface;
use Drupal\Core\Menu\MenuTreeParameters;
use Drupal\Core\Theme\ThemeAccessCheck;
use Drupal\Core\Url;
use Drupal\system\SystemManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Serializer\Exception\UnexpectedValueException;


/**
 * Provides route responses for the hello world page example.
 */
class DroogleController {
  /**
   * Retrieves a configuration object.
   *
   * This is the main entry point to the configuration API. Calling
   * @code $this->config('book.admin') @endcode will return a configuration
   * object in which the book module can store its administrative settings.
   *
   * @param string $name
   *   The name of the configuration object to retrieve. The name corresponds to
   *   a configuration file. For @code \Drupal::config('book.admin') @endcode,
   *   the config object returned will contain the contents of book.admin
   *   configuration file.
   *
   * @return \Drupal\Core\Config\Config
   *   A configuration object.
   */
  protected function config($name) {
    if (!$this->configFactory) {
      $this->configFactory = $this->container()->get('config.factory');
    }
    return $this->configFactory->get($name);
  }

  /**
   * Returns the service container.
   *
   * This method is marked private to prevent sub-classes from retrieving
   * services from the container through it. Instead,
   * \Drupal\Core\DependencyInjection\ContainerInjectionInterface should be used
   * for injecting services.
   *
   * @return \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container.
   */
  private function container() {
    return \Drupal::getContainer();
  }

  /**
   * Returns a simple hello world page.
   *
   * @return array
   *   A very simple renderable array is returned.
   */
  public function droogleCallbackMethod() {
    $element = array(
      '#markup' => '<p><b>Saying Hello World in Drupal 8 is cool!  Here are the beginnings of the Droogle module!!!!!</b></p>',
    );
    return $element;
  }
  /**
   * Returns a list of Droogle Drive Documents.
   *
   * @return array
   *   A very simple renderable array is returned.
   */
  public function droogleDriveCallbackMethod() {
    $folder_list = $this->droogle_drive_folderlist();
    // return $folder_list;
    \Drupal::logger('my_module')->notice('<pre>' . print_r($folder_list, TRUE) . '</pre>');
    $element = array(
      '#markup' => $folder_list,
      '#attached' => array(
        'css' => array(
          drupal_get_path('module', 'droogle') . '/css/droogle_drive_list.css',
          drupal_get_path('module', 'droogle') . '/css/droogle_drive.css',
        ),
        'js' => array(drupal_get_path('module', 'droogle') . '/js/droogle_drive_list.js'),
      ),
    );
    return $element;

  }

  public function droogleDriveSharedListCallbackMethod() {
    $user = \Drupal::currentUser();
    $config = $this->config('droogle.settings');
    $droogle_drive_title_text = $config->get('cache.droogle_drive_title_text');
    $user_email = $user->getEmail();
    $file_id = $_POST['fileId'];

    if ($droogle_drive_title_text == "") {
      $droogle_drive_title_text = $config->get('cache.droogle_title_text');
    }

    unset($result);
    $return_result = $this->droogle_drive_gdrive_connect('droogle_drive');
    $client = $return_result['client'];
    $service = $return_result['service'];
    if (isset($return_result['authUrl'])) {
      $authUrl = $return_result['authUrl'];
    }
    // If we're signed in then lets try to upload our file.
    $refresh_token = $config->get('cache.droogle_drive_refresh_token');
    if ($refresh_token == "") {
      $refresh_token = $config->get('cache.droogle_refresh_token');
    }
    if(!empty($refresh_token) && $client->isAccessTokenExpired()) {
      $client->refreshToken($refresh_token);
    }

    if ($client->getAccessToken()) {
      $content = $this->droogleDriveSharedListing($service, $user_email, $file_id);
    }
    elseif (isset($authUrl)) {
      $content = '<a class="login" href=' . $authUrl . '>Connect Me!</a></div>';
    }
    /*$element = array(
      '#attached' => array(
        'css' => array(drupal_get_path('module', 'droogle') . '/css/droogle_drive_list.css'),
        'js' => array(drupal_get_path('module', 'droogle') . '/js/droogle_drive_list.js'),
      ),
    );
    render($element);*/

    return new JsonResponse(array(
      'successful' => 'Droogle Block Returned',
      'doclink' => $content,
    ));
    /*$serializer = $this->container->get('serializer');
    $output = $serializer->serialize($content, 'json');
    return $output;*/
  }

  /**
   * Returns a list of GOogle Drive documents.
   *
   */
  protected function droogle_drive_folderlist() {
    $user = \Drupal::currentUser();
    $config = $this->config('droogle.settings');
    $droogle_drive_title_text = $config->get('cache.droogle_drive_title_text');
    // $owner_email = $_POST['ownerEmail'];
    $owner_email = $user->getEmail();


    if ($droogle_drive_title_text == "") {
      $droogle_drive_title_text = $config->get('cache.droogle_title_text');
    }

    unset($result);
    $return_result = $this->droogle_drive_gdrive_connect('droogle_drive');
    $client = $return_result['client'];
    $service = $return_result['service'];
    if (isset($return_result['authUrl'])) {
      $authUrl = $return_result['authUrl'];
    }
    // If we're signed in then lets try to upload our file.
    $refresh_token = $config->get('cache.droogle_drive_refresh_token');
    if ($refresh_token == "") {
      $refresh_token = $config->get('cache.droogle_refresh_token');
    }
    if(!empty($refresh_token) && $client->isAccessTokenExpired()) {
      $client->refreshToken($refresh_token);
    }

    // return new JsonResponse($content);
    //if ($client->getAccessToken()) {
    /*return array(
      '#theme' => 'droogle_list_folders',
      'service' => $service,
      'owner_email' => $owner_email,
    );*/


    if ($client->getAccessToken()) {
    $content = $this->droogle_list_files($service, $owner_email);
    }
  elseif (isset($authUrl)) {
      $content = '<a class="login" href=' . $authUrl . '>Connect Me!</a></div>';
    }

  /*drupal_json_output(array(
    'successful' => 'success!',
    'folderLink' => $content,
  ));*/

  return $content;

  }

  /*
   *  Function connects to google drive.
   */
  protected function droogle_drive_gdrive_connect($redirect_uri = NULL) {
    $config = $this->config('droogle.settings');
    if ($path = libraries_get_path('google-api-php-client-git-version')) {
      ini_set('include_path',ini_get('include_path').':' . $_SERVER['DOCUMENT_ROOT'] . '/sites/all/libraries/google-api-php-client-git-version/src');
      require_once $path . '/src/Google/Client.php';
      require_once $path . '/src/Google/Http/MediaFileUpload.php';
      require_once $path . '/src/Google/Service/Drive.php';
    }
    else {
      drupal_set_message('You need to install the google api client library in the folder
      google-api-php-client-git-version within the sites/all/libraries folder');
    }


    /************************************************
    ATTENTION: Fill in these values! Make sure
    the redirect URI is to this page, e.g:
    http://localhost:8080/fileupload.php
     ************************************************/
    if (\Drupal::moduleHandler()->moduleExists('og_context')) {
      $group_google_clientid_cck = 'field_' . $config->get('cache.droogle_drive_og_cck_field_gdrive_client_id');
      $group_google_clientsecret_cck = 'field_' . $config->get('cache.droogle_drive_og_cck_field_gdrive_client_secret');
      $current_group = og_context();
      if (isset($current_group->etid)) {
        $group_node = node_load($current_group->etid);
        $client_id = $group_node->{$group_google_clientid_cck}['und'][0]['value'];
        $client_secret = $group_node->{$group_google_clientsecret_cck}['und'][0]['value'];
      }
    }

    if (!isset($droogle_drive_clientid) || !isset($droogle_drive_secret)) {
      $droogle_drive_clientid = $config->get('cache.droogle_drive_clientid');
      if ($droogle_drive_clientid == "") {
        $droogle_drive_clientid = $config->get('cache.droogle_clientid');
      }
      $droogle_drive_secret = $config->get('cache.droogle_drive_secret');
      if ($droogle_drive_secret == "") {
        $droogle_drive_secret = $config->get('cache.droogle_secret');
      }
    }
    $client = new \Google_Client();
    $client->setClientId($droogle_drive_clientid);
    $client->setClientSecret($droogle_drive_secret);
//  $client->setRedirectUri('http://' . $_SERVER['HTTP_HOST'] . '/' . $redirect_uri);
    $client->setRedirectUri('http://' . $_SERVER['HTTP_HOST'] . '/droogle-drive');
    $client->addScope("https://www.googleapis.com/auth/drive");
    $service = new \Google_Service_Drive($client);

    if (isset($_REQUEST['logout'])) {
      unset($_SESSION['upload_token ']);
    }

    if (isset($_GET['code'])) {
      $client->authenticate($_GET['code']);
      $_SESSION['upload_token'] = $client->getAccessToken();
      $redirect = 'http://' . $_SERVER['HTTP_HOST'] . $_SERVER['PHP_SELF'];
      header('Location: ' . filter_var($redirect, FILTER_SANITIZE_URL));
    }

    if (isset($_SESSION['upload_token']) && $_SESSION['upload_token']) {
      $client->setAccessToken($_SESSION['upload_token']);
      if ($client->isAccessTokenExpired()) {
        unset($_SESSION['upload_token']);
      }
    } else {
      $authUrl = $client->createAuthUrl();
    }
    $return_result['client'] = $client;
    $return_result['service'] = $service;
    if (isset($authUrl)) {
      $return_result['authUrl'] = $authUrl;
    }
    return $return_result;
  }

  protected function droogle_list_files($service, $owner_email) {
    $user_email = $owner_email;

    $files = $service->files->listFiles();

    $file_list = $files->getItems();

    $owner_content = '';
    // Below line if for debugging.
    // $content .= '<div class="shortened">File List:  <pre> ' . print_r($file_list, TRUE) . '</pre></div></div>';
    foreach ($file_list as $file) {
      $owner_email = $file['data']['owners'][0]['emailAddress'];
      $fileId = $file['id'];

      if ($owner_email == $user_email) {
        $owner_content .= '<li class="droogle-owned-doc-list">';
        $owner_content .=   '<a href="' . $file["alternateLink"] . '">';
        $owner_content .=     '<img class="droogle-drive-doc-file-icon" src="' . $file["iconLink"] . '">' . $file["title"];
        $owner_content .=   '</a>';
        $owner_content .=   '<button class="droogle-document-get-shared-list">';
        $owner_content .=     'Show Sharing';
        $owner_content .=     '<span class="hidden droogle-document-shared-list-fileid">' . $fileId . '</span>';
        $owner_content .=     '<span class="hidden droogle-document-shared-list-useremail">' . $user_email . '</span>';
        $owner_content .=    '</button>';
        $owner_content .=   '<button class="please-standby get-doc-list">';
        $owner_content .=     'Please Standby';
        $owner_content .=    '</button>';
        $owner_content .=   '<button class="droogle-document-hide-shared-list">';
        $owner_content .=     'Hide Sharing';
        $owner_content .=    '</button>';
        $owner_content .=   '<button class="droogle-document-show-shared-list">';
        $owner_content .=     'Show Sharing';
        $owner_content .=    '</button>';
        $owner_content .=    '<div class="droogle-drive-shared-list-links-wrapper"></div>';
        $owner_content .= '</li>';
      }
      /* Getting the shared files from the following code takes too long so we won't use it, but still don't want to delete it either */
      /*
          else {
            $shared_list = droogle_drive_list_retrievePermissions($service, $fileId);
            foreach ($shared_list as $shared_person) {
              $shared_email = $shared_person['emailAddress'];
              if ($shared_email == $user_email && (!isset($file['explicitlyTrashed']) && $file['explicitlyTrashed'] != 1)) {
                $shared_content .= '<a href="' . $file["alternateLink"] . '"><div><img class="droogle-drive-doc-file-icon" src="' . $file["iconLink"] . '">' . $file["title"] . '</div></a>';
              }
            }
          }
      */
//    $content .= '<a href="' . $file["alternateLink"] . '"><div><img class="droogle-drive-doc-file-icon" src="' . $file["iconLink"] . '">' . $file["title"] . '</div></a>';
    }

    $content = '<ul id="droogle-drive-document-listing">';
    $content .= '<li id="droogle-owner-content">Shared Google Docs Owned by You</li>';
    $content .= $owner_content;
//  $content .= '<li id="droogle-shared-content">Shared with You</li>';
//  $content .= $shared_content;
    $content .= '</ul>';

    return $content;

}

  protected function droogle_list_folders($service, $owner_email) {
    $ajax_owner_email = $owner_email;

    $files = $service->files->listFiles();

    $file_list = $files->getItems();


    $owner_content = '';
    \Drupal::logger('my_module')->error('<pre>' . print_r($file_list, TRUE) . '</pre>');
    foreach ($file_list as $file) {
      $fileId = $file['id'];
      if ($file['mimeType'] != 'application/vnd.google-apps.folder' || (isset($file['explicitlyTrashed']) && $file['explicitlyTrashed'] == 1)) {
        //Only list folders, just files (for now). It's displaying all files in folders anyhow.
        continue;
      } else {
        // get owner info from google drive folder and compare
        $owner_email = $file['data']['owners'][0]['emailAddress'];
        if ($owner_email == $ajax_owner_email) {
          $owner_content .= '<li class="droogle-owned-folder-list">';
//        $owner_content .=   '<a href="' . $file["alternateLink"] . '">';
          $owner_content .=     '<img class="droogle-drive-doc-file-icon" src="' . $file["iconLink"] . '">' . $file["title"];
//        $owner_content .=   '</a>';
          $owner_content .=   '<span class="select-status"></span>';
          $owner_content .=   '<span class="hidden droogle-document-shared-list-fileid">' . $fileId . '</span>';
          $owner_content .= '</li>';

        }
      }
    }
    $content =  '<ul class="droogle-drive-folder-listing">';
    $content .=   '<li id="droogle-folder-owner-list-title" class="expanded">Select Destination Folder (optional)</li>';
    $content .=   '<li class="droogle-owned-folder-list my-drive selected">My Drive (default)<span class="select-status">selected</span></li>';
    $content .=    $owner_content;
    $content .= '</ul>';


    return $content;
  }

  protected function DroogleDriveSharedListing($service, $user_email, $file_id) {
    $shared_list = $this->droogleDriveListRetrievePermissions($service, $file_id);
    $shared_content = '';
    foreach ($shared_list as $shared_person) {
      $shared_email = $shared_person['emailAddress'];
      $shared_name = $shared_person['name'];
      $shared_role = $shared_person['role'];
      if ($shared_email != NULL) {
//      if ($user_email != $shared_email) {
        if ($shared_name == NULL) {
          $shared_name = $shared_email;
        }
        if (\Drupal::moduleHandler()->moduleExists('droogle_hangouts')) {
          // This needs to be updated for Drupal 8
          $email_button = $this->droogle_hangout_email_button($shared_email, FALSE);
        }
        if (\Drupal::moduleHandler()->moduleExists('droogle')) {
          $droogle_button = $this->droogle_drive_create_invite_button($shared_email, $shared_name, FALSE);
        }
        if (\Drupal::moduleHandler()->moduleExists('jabber')) {
          // This needs to be updated for Drupal 8
          $chat_button = $this->droogle_hangout_chat_button($shared_email, $shared_name, FALSE);
        }

        if (\Drupal::moduleHandler()->moduleExists('droogle_hangouts')) {
          // This needs to be updated for Drupal 8
          $hangout_button = $this->droogle_hangout_invite_button($shared_email, $shared_name, FALSE);
        }

        switch ($shared_role) {
          case 'owner':
            $shared_label = 'Is owner';
            break;
          case 'writer':
            $shared_label = 'Can edit';
            break;
          case 'reader':
            $shared_label = 'Can view';
            break;
        }

        $shared_content .= '<li class="droogle-shared-list-item">';
        $shared_content .=   '<span  class="droogle-shared-list-name">';
        $shared_content .=     $shared_name;
        $shared_content .=   '</span>';
        $shared_content .=   '<span class="droogle-shared-list-name-role">';
        $shared_content .=     ': ' . $shared_label;
        $shared_content .=   '</span>';
        $shared_content .=   '<span class="droogle-button-array">';
        //$shared_content .=     $chat_button;
        //$shared_content .=     $email_button;
        //$shared_content .=     $hangout_button;
        $shared_content .=     $droogle_button;
        $shared_content .=    '<span class="droogle-button-array-last"></span>';
        $shared_content .=   '</span>';
        $shared_content .= '</li>';
//      }
      }
    }
    $content =  '<ul class="droogle-shared-list">';
    $content .= '<li class="droogle-shared-list-title">This Document is Shared With<div class="close-droogle-shared-list">x</span></span></li>';
    $content .= $shared_content;
    $content .= '</ul>';
    return $content;
  }
  /**
   * Retrieve a list of permissions.
   *
   * @param Google_DriveService $service Drive API service instance.
   * @param String $fileId ID of the file to retrieve permissions for.
   * @return Array List of permissions.
   */
  protected function droogleDriveListRetrievePermissions($service, $fileId) {
    try {
      $permissions = $service->permissions->listPermissions($fileId);
      return $permissions->getItems();
    } catch (Exception $e) {
      print "An error occurred: " . $e->getMessage();
    }
    return NULL;
  }

  /**
   *  Implements hook_theme() for Create Doc invite, create and google docs.
   *  Phineas, isn't using a theme function for a button, Overkill?
   *  Why yes Mr. Drupal Developer, yes it is.
   */

  protected function droogle_drive_create_invite_button($shared_email, $fullname, $text) {

    if ($text == TRUE) {
      $text = 'Google Drive';
    } else {
      $text = '';
    }

    if (is_null($fullname) || $fullname =='') {
      $fullname = $email;
    }
    $content  = '<div class="droogle-drive-create-invite-btn-wrapper">';
    $content .=   '<button class="droogle-drive-create-invite-btn" name="' . $email . '">' . $text;
    $content .=     '<span class="button-fullname" style="display:none;">' . $fullname . '</span>';
    $content .=     '<span class="text-balloon-wrapper"><span class="text-balloon">';
    $content .=       'google drive';
    $content .=     '</span></span>';
    $content .=	'</button>';
    $content .= '</div><!--#droogle-drive-doc-invite-btn-wrapper -->';
    return $content;
  }

  protected function droogle_drive_create_invite_all_button($shared_email, $shared_name, $text) {
    if ($text == TRUE) {
      $text = 'Create and Share Google Doc';
    } else {
      $text = '';
    }
    $fullname = $variables['fullname'];
    if (is_null($fullname) || $fullname =='') {
      $fullname = $email;
    }
    $content  = '<div class="droogle-drive-create-invite-all-btn-wrapper">';
    $content .=   '<button class="droogle-drive-create-invite-all-btn" name="' . $email . '">' . $text;
    $content .=     '<span class="button-fullname" style="display:none;">' . $fullname . '</span>';
    $content .=     '<span class="text-balloon-wrapper"><span class="text-balloon">';
    $content .=       'google drive';
    $content .=     '</span></span>';
    $content .=	'</button>';
    $content .= '</div><!--#droogle-drive-doc-invite-btn-wrapper -->';

    return $content;
  }
}
