<?php
 
/**
 * @file
 * Contains \Drupal\hello_world\Form\DroogleAdminForm.
 */
 
namespace Drupal\droogle\Form;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a simple example form.
 */
class DroogleAdminForm extends ConfigFormBase {
 
  /**
   * Implements \Drupal\Core\Form\FormInterface::getFormID().
   */
  public function getFormID() {
    return 'droogle_settings_form';
  }

  /**
   * Implements \Drupal\Core\Form\FormInterface::buildForm().
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    global $user;
    $config = $this->config('droogle.settings');

    $form['droogle_default_domain'] = array(
      '#type' => 'textfield',
      '#title' => t('Enter the default domain to use to form links to your google docs ie: babson.edu, if your email address is simply yourname@gmail.com then do not worry about this, but if you are using a google education or business account and your gmail address is yourname@babson.edu for instance then enter a value for this field'),
      '#default_value' => $config->get('cache.droogle_default_domain'),
      '#size' => 60,
      '#maxlength' => 64,
      '#description' => t('the default domain to use in creating links to your google docs'),
    );
    $form['sitewide_nonog'] = array(
      '#type' => 'fieldset',
      '#title' => t('Settings controlling the non Organic Groups context for Droogle'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['sitewide_nonog']['droogle_title_text'] = array(
      '#type' => 'textfield',
      '#title' => t('Enter the title to put at the top of the droogle page (when not within an Organic Groups context)'),
      '#default_value' => $config->get('cache.droogle_title_text'),
      '#size' => 60,
      '#maxlength' => 64,
      '#description' => t('the non OG title of the droogle page, default is:  "DROOGLE: A list of your google docs"'),
    );
    $form['sitewide_nonog']['droogle_clientid'] = array(
      '#type' => 'textfield',
      '#title' => t('Enter the google client id to use (when not within an Organic Groups context)
                  -- you can generate a client id at https://cloud.google.com/console'),
      '#default_value' => $config->get('cache.droogle_clientid'),
      '#size' => 100,
      '#maxlength' => 150,
      '#description' => t('the non OG google client id'),
    );
    $form['sitewide_nonog']['droogle_secret'] = array(
      '#type' => 'textfield',
      '#title' => t('Enter the google client secret to use (when not within an Organic Groups context)
                  -- you can generate a client secret at https://cloud.google.com/console'),
      '#default_value' => $config->get('cache.droogle_secret'),
      '#size' => 60,
      '#maxlength' => 64,
      '#description' => t('the non OG google client secret'),
    );
    $form['sitewide_nonog']['droogle_refresh_token'] = array(
      '#type' => 'textfield',
      '#title' => t('Enter the google refresh token, generate one by going to
      the url http://yoururl.com/droogle_get_google_token but MAKE SURE TO ENABLE THE DEVEL MODULE FIRST,
      you can disable the devel module after you generate the refresh token, ignore the error and
      copy and paste the refesh token generated (not the access code) into this text box'),
      '#default_value' => $config->get('cache.droogle_refresh_token'),
      '#size' => 60,
      '#maxlength' => 64,
      '#description' => t('the google refresh token so the login to google does not timeout'),
    );
    if (\Drupal::moduleHandler()->moduleExists('og')) {
      $form['single_groups'] = array(
        '#type' => 'fieldset',
        '#title' => t('Overriding google username and password for individual Organic Group(s)'),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
      );
      $form['single_groups']['droogle_og_cck_field_gdrive_client_id'] = array(
        '#type' => 'textfield',
        '#title' => t('Enter the "group" content types cck field machine name to use to override
                    a particular groups google client id)'),
        '#default_value' => $config->get('cache.droogle_og_clientid_cck_field'),
        '#size' => 60,
        '#maxlength' => 64,
        '#description' => t('the OG group cck field to use to set a particular OG droogle password
                          -- you will have to setup a new cck field for the "group" content type
                          matching the machine name set on this page'),
      );
      $form['single_groups']['droogle_og_cck_field_gdrive_client_secret'] = array(
        '#type' => 'textfield',
        '#title' => t('Enter the "group" content types cck field machine name to use to override
                    a particular groups google client secret -- this is optional and if the field
                    is left empty when configuring a particular group, the global Droogle organic
                    groups settings will be followed)'),
        '#default_value' => $config->get('cache.droogle_og_clientsecret_cck_field'),
        '#size' => 60,
        '#maxlength' => 64,
        '#description' => t('the OG group cck field to use to set a particular OG droogle password
                          -- you will have to setup a new cck field for the "group" content type
                          matching the machine name set on this page'),
      );
    }
    return parent::buildForm($form, $form_state);

  }
 
  /**
   * Implements \Drupal\Core\Form\FormInterface::validateForm().
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

 }
 
  /**
   * Implements \Drupal\Core\Form\FormInterface::submitForm().
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('droogle.settings')
      ->set('cache.droogle_clientid', $form_state->getValue('droogle_clientid'))
      ->set('cache.droogle_secret', $form_state->getValue('droogle_secret'))
      ->set('cache.droogle_title_text', $form_state->getValue('droogle_title_text'))
      ->set('cache.droogle_password', $form_state->getValue('droogle_password'))
      ->set('cache.droogle_refresh_token', $form_state->getValue('droogle_refresh_token'))
      ->set('cache.droogle_og_cck_field_gdrive_client_id', $form_state->getValue('droogle_og_cck_field_gdrive_client_id'))
      ->set('cache.droogle_og_cck_field_gdrive_client_secret', $form_state->getValue('droogle_og_cck_field_gdrive_client_secret'))
      ->set('cache.droogle_default_domain', $form_state->getValue('droogle_default_domain'))
      ->save();
    $msg= "Thank you, your form has been submitted successfully";
    drupal_set_message($msg);

  }

}
